#include "Default.h"

#include <cairomm/context.h>
#include <math.h>
#include <string>
#include <iostream>
using namespace std;

//Variables
const string gauges[3] = {"speed", "rpm", "fuel"};

const double gaugeRadius = 0.225;
const double gaugeWidth = 0.45;
const double gaugePadding = 0.05;
const double gaugeLowerPadding = 0.10; //percentages/100
const double signalArea = 0.15; //15% of the main drawing area will be devoted tCairo::RefPtr<Cairo::Context> cro drawign the signals
const double drawableArea = 0.8;  //80% of the drawable area will be devoted to drawing the gauges
const double maxSpeed = 120;
const double maxRPM = 8000;



//Methods
Default::Default(void) {
    currentSpeedValue = 0;
    currentRpmValue = 0;
    currentFuelValue = 0;
    initialize = true;

    file_read = Glib::IOChannel::create_from_file("test.txt", "r");
    Glib::signal_io().connect(sigc::mem_fun(*this, &Default::Update), file_read, Glib::IO_IN | Glib::IO_HUP, G_PRIORITY_DEFAULT_IDLE);
}

Default::~Default(void) {
}

bool Default::Update(Glib::IOCondition io_condition) { 
    Glib::ustring buf;
    cout << "created the buffer\n";

    Glib::IOStatus status = file_read->read_line(buf);
    if (status == Glib::IO_STATUS_NORMAL) {
        cout << "read in a line " << buf << "\n";

        if (!buf.empty()) {
            string temp = buf.raw();
            string tempVal = temp.substr(2, temp.length()-3);
            temp.erase(1, temp.length()-1);
            sensor_to_process = static_cast<SensorID>(atoi(temp.c_str()));
            sensor_value = atof(tempVal.c_str());
            cout << "data received\n";
            // force the program to redraw the screen
            queue_draw();
            cout << "bye\n\n";
        }
        return true;
    }
    // there was an error or EOF, disconnect this signal handler
    return false;
}

// use this to draw the intial screen
bool Default::on_expose_event(GdkEventExpose* event) {
    cout << "exposing\n";
    Glib::RefPtr<Gdk::Window> window = get_window();
    if(window) {
        Gtk::Allocation allocation = get_allocation();
        totalDrawableWidth = allocation.get_width();
        totalDrawableHeight = allocation.get_height();
       
        cr = window->create_cairo_context();   
        cr->rectangle(event->area.x, event->area.y, event->area.width, event->area.height);
        cr->clip();
       
        if(!initialize) {
            switch(sensor_to_process) {
                case 0:
                    cout << "speed baby\n";
                    processSpeed(sensor_value);
                    break;
                case 1:
                    processRPM(sensor_value);
                    break;
                case 2:
                    processFuel(sensor_value);
                    break;
                default:
                    cerr << "no luck bud\n";
                    break;
            }
        }
        else {
            // Draw initial gauges
            drawGauge(gaugePadding*totalDrawableWidth);  // Speed Gauge
            drawGauge((gaugeWidth + gaugePadding)*totalDrawableWidth);  // RPM Gauge

            drawNeedle(180, gaugePadding*totalDrawableWidth);  // Speed Needle
            drawNeedle(180, (gaugeWidth + gaugePadding)*totalDrawableWidth);  // RPM Needle

            drawFuelGauge(0);
            initialize = false;
            cout << "DONE INTIALIZATION\n";
        }
    }
    cr.clear();
    return true;
}

void Default::drawGauge(double Offset) {
    cr->set_source_rgb(0,0,0);
    cr->set_line_width(5);
    cr->set_line_join(Cairo::LINE_JOIN_ROUND);
   
    //draw gauge outlines
    cr->save();
    cr->arc(gaugeRadius * totalDrawableWidth + Offset + 1, totalDrawableHeight - (gaugePadding * totalDrawableHeight), gaugeRadius * totalDrawableWidth, M_PI, 0);
    cr->stroke();   
    cr->restore();

    // Draw tick marks   
    double lineLength = 0;
    bool evens = true;
    int temp = 0;
    int spacer = 0;
    for(int x = 0; x<13; x++) {
        cr->save();
        cr->set_line_join(Cairo::LINE_JOIN_ROUND);
        cr->set_line_width(3);
        spacer = 0;
       
        if(x%2 == 0) {
            lineLength = gaugePadding * totalDrawableWidth;
            evens = true;
        }
        else {
            lineLength = gaugePadding * totalDrawableWidth/2;
            evens = false;
        }
           
        cr->move_to(Offset + gaugeRadius * totalDrawableWidth + 1 + (gaugeRadius * totalDrawableWidth - lineLength) * cos(x*M_PI/12), totalDrawableHeight - (gaugePadding * totalDrawableHeight) - (gaugeRadius * totalDrawableWidth - lineLength) * sin(x*M_PI/12));
        cr->line_to(Offset + gaugeRadius * totalDrawableWidth + 1 + (gaugeRadius * totalDrawableWidth) * cos(x*M_PI/12), totalDrawableHeight - (gaugePadding * totalDrawableHeight) - (gaugeRadius * totalDrawableWidth) * sin(x*M_PI/12));
       
        // write values of even tick marks to the screen
        if(evens) {
            string s;
            stringstream out;
            temp = 120 - 20*x/2;
            out << temp;
            s = out.str();
            if(temp >=100) {
                spacer = 35;
            }
            if(temp >=60 && temp <=80) {
                spacer = 10;
            }
            cr->move_to(Offset + gaugeRadius * totalDrawableWidth + 1 + (gaugeRadius * totalDrawableWidth - lineLength - 7 - spacer) * cos(x*M_PI/12), totalDrawableHeight - (gaugePadding * totalDrawableHeight) - (gaugeRadius * totalDrawableWidth - lineLength - spacer - 7) * sin(x*M_PI/12));
            cr->set_font_size(15);
            cr->text_path(s); 
        }
       
        cr->stroke();
        cr->restore();
    }
    //cout << "here\n";
    //cr->stroke();
    //cout <<"done here\n";
}

void Default::drawFuelGauge(double Nangle) {
   
    // Process angle
    Nangle = Nangle * M_PI / 180;

    cr->set_source_rgb(0,0,0);
    cr->set_line_width(5);

    cr->save();
    cr->arc((2*gaugeRadius + gaugePadding) * totalDrawableWidth + 1, totalDrawableHeight - ((gaugePadding + 2*gaugeRadius)* totalDrawableHeight), gaugeRadius/2 * totalDrawableWidth, M_PI, 0);
    cr->stroke();   
    cr->restore();
   
    // Draw tick marks   
    double lineLength = 0;
    bool evens = true;
    int count = 0;
    int temp = 0;
    int spacer = 0;
    string labels[] = {"F","3/4","1/2","1/4","E"};
    for(int x = 0; x<5; x++) {
        cr->save();
        cr->set_line_width(3);
        spacer = 0;
       
        if(x%2 == 0) {
            lineLength = gaugePadding * totalDrawableWidth;
            evens = true;
           
        }
        else {
            lineLength = gaugePadding * totalDrawableWidth/2;
            evens = false;
        }
           
        cr->move_to((2*gaugeRadius + gaugePadding) * totalDrawableWidth + 1 + (gaugeRadius/2 * totalDrawableWidth - lineLength) * cos(x*M_PI/4), totalDrawableHeight - ((gaugePadding + 2*gaugeRadius)* totalDrawableHeight) - (gaugeRadius/2 * totalDrawableWidth - lineLength) * sin(x*M_PI/4));
        cr->line_to((2*gaugeRadius + gaugePadding) * totalDrawableWidth + 1 + (gaugeRadius/2 * totalDrawableWidth) * cos(x*M_PI/4), totalDrawableHeight - ((gaugePadding + 2*gaugeRadius)* totalDrawableHeight) - (gaugeRadius/2 * totalDrawableWidth) * sin(x*M_PI/4));
       
        cr->stroke();
        cr->restore();
    }
   
    //Draw Needle
    //Draw center dot
    cr->save();
    cr->set_source_rgba(1,0.2,0.2,1);   
    cr->arc((2*gaugeRadius + gaugePadding) * totalDrawableWidth + 1, totalDrawableHeight - ((gaugePadding + 2*gaugeRadius)* totalDrawableHeight), gaugePadding/6*totalDrawableWidth, 0, 2*M_PI);
    cr->fill();
    cr->stroke();
    cr->restore();

    // Draw needle line
    cr->save();
    cr->set_source_rgba(1,0.2,0.2,0.6);   
    cr->set_line_width(6);
    cr->set_line_cap(Cairo::LINE_CAP_ROUND);
    cr->move_to((2*gaugeRadius + gaugePadding) * totalDrawableWidth + 1, totalDrawableHeight - ((gaugePadding + 2*gaugeRadius)* totalDrawableHeight));
    cr->line_to((2*gaugeRadius + gaugePadding) * totalDrawableWidth + 1 + (gaugeRadius/2 * totalDrawableWidth -gaugePadding/2*totalDrawableWidth) * cos(Nangle), totalDrawableHeight - ((gaugePadding + 2*gaugeRadius)* totalDrawableHeight) - (gaugeRadius/2 * totalDrawableWidth - gaugePadding/2*totalDrawableWidth) * sin(Nangle));
    cr->stroke();
    cr->restore();
   
}

void Default::drawNeedle(double Nangle, double Offset) {
    // Draw center dot   
    cr->save();
    cr->set_source_rgba(1,0.2,0.2,1);   
    cr->arc(gaugeRadius * totalDrawableWidth + Offset + 1, totalDrawableHeight - (gaugePadding * totalDrawableHeight), gaugePadding/6*totalDrawableWidth, 0, 2*M_PI);
    cr->fill();
    cr->stroke();
    cr->restore();

    // Process angle
    Nangle = Nangle * M_PI / 180;

    // Draw needle line
    cr->save();
    cr->set_source_rgba(1,0.2,0.2,0.6);   
    cr->set_line_width(6);
    cr->set_line_cap(Cairo::LINE_CAP_ROUND);
    cr->move_to(gaugeRadius * totalDrawableWidth + Offset + 1, totalDrawableHeight - (gaugePadding * totalDrawableHeight));
    cr->line_to(Offset + gaugeRadius * totalDrawableWidth + 1 + (gaugeRadius * totalDrawableWidth -gaugePadding/2*totalDrawableWidth) * cos(Nangle), totalDrawableHeight - (gaugePadding * totalDrawableHeight) - (gaugeRadius * totalDrawableWidth - gaugePadding/2*totalDrawableWidth) * sin(Nangle));
    cr->stroke();
    cr->restore();
}

void Default::processSpeed(double value) {
    cout << "updating speed\n";
    //cr->rectangle(gaugePadding * totalDrawableWidth, gaugeLowerPadding * totalDrawableHeight, gaugeWidth * totalDrawableWidth, gaugeRadius * totalDrawableHeight);
    //cr->clip();   
    currentSpeedValue = value;
    double Nangle = 0;
   
    if(value <= 120) {
        cout << "I am here - 120\n";
        Nangle = (1-(value/120)) * M_PI;  // Already in radians
    }
    else {
        cout << "I am here - 0\n";
        Nangle = 0;
    }
    cout << "the angle is: " << Nangle << endl;
    cout << "Drawing Gauge\n";
    drawGauge(gaugePadding * totalDrawableWidth);
    cout << "Drawing Needle\n";
    drawNeedle(Nangle, 0);
    cout << "DONE!!!!!!!!!\n";
}

void Default::processRPM(double value) {
    //cr->rectangle((gaugePadding + gaugeWidth) * totalDrawableWidth, gaugeLowerPadding * totalDrawableHeight, gaugeWidth * totalDrawableWidth, gaugeRadius * totalDrawableHeight);
    //cr->clip();   
    currentRpmValue = value;
    double Nangle = 0;
   
    if(value <= 8000)
        Nangle = (1 - (value/8000)) * M_PI; // Already in radians
    else
        Nangle = 0;
   
    drawGauge((gaugePadding + gaugeWidth) * totalDrawableWidth);
    drawNeedle(Nangle, (gaugePadding + gaugeWidth) * totalDrawableWidth);
}

void Default::processFuel(double value) {
    currentFuelValue = value;
    double Nangle = 0;
   
    if(value <= 100)
        Nangle = (1 - (value/100)) * M_PI; // Already in radians
    else
        Nangle = 0;
   
    drawFuelGauge(Nangle);
}

